import sys
import collections
from pysnmp.entity.rfc3413.oneliner import cmdgen

def datafrommib(mib, community, ip):
    value = tuple([int(i) for i in mib.split('.')])
    generator = cmdgen.CommandGenerator()
    comm_data = cmdgen.CommunityData('server', community, 1) # 1 means version SNMP v2c
    transport = cmdgen.UdpTransportTarget((ip, 161))

    real_fun = getattr(generator, 'nextCmd')
    res = (errorIndication, errorStatus, errorIndex, varBindTable)\
            = real_fun(comm_data, transport, value)

    if not errorIndication is None  or errorStatus is True:
           print "Error: %s %s %s %s" % res
           yield None
    else:
        for varBindTableRow in varBindTable:
            data = varBindTableRow[0]
            port = data[0]._value[len(value):]
            octets = data[1]

            yield {'port': port[0], 'octets': octets}

def status(ip, community):
    # for use snmptool try:
    # snmpwalk -c mymypub -v2c <ip> <mib>
    # e.t.c...
    mibs = [('1.3.6.1.2.1.2.2.1.8', 'ifOperStatus'),
            ('1.3.6.1.2.1.2.2.1.3', 'ifType'),
            ('1.3.6.1.2.1.2.2.1.5', 'ifSpeed'),
            ('1.3.6.1.2.1.31.1.1.1.1', 'ifName'),
            ('1.3.6.1.2.1.31.1.1.1.18', 'ifAlias')
            ]

    ports = collections.defaultdict(dict)

    for mib in mibs:
        data = datafrommib(mib[0], community, ip)
        for row in data:
            if row:
                ports[row['port']][mib[1]] = row['octets']
            else:
                return None

    return ports

if __name__ == '__main__':
    try:
        ip = sys.argv[1]
        community = sys.argv[2]
    except IndexError:
        print "Please run command like:"
        print "python %s <ip> <community>" % (__file__)
        sys.exit(0)

    ports = status(ip, community)
    if ports:
        for key, value in ports.items():
            print key, ('ifOperStatus: %(ifOperStatus)s ifType: %(ifType)s' +\
                        ' ifSpeed: %(ifSpeed)s ifName: %(ifName)s' +\
                        ' ifAlias: %(ifAlias)s') % value




import sys
from pysnmp.entity.rfc3413.oneliner import cmdgen

def fetchFdb(ip, community):
    mib = '1.3.6.1.2.1.17.7.1.2.2.1.2'
    value = tuple([int(i) for i in mib.split('.')])
    generator = cmdgen.CommandGenerator()
    comm_data = cmdgen.CommunityData('server', community, 1) # 1 means version SNMP v2c
    transport = cmdgen.UdpTransportTarget((ip, 161))

    real_fun = getattr(generator, 'nextCmd')
    res = (errorIndication, errorStatus, errorIndex, varBindTable)\
        = real_fun(comm_data, transport, value)

    if not errorIndication is None  or errorStatus is True:
           print "Error: %s %s %s %s" % res
    else:
        for varBindTableRow in varBindTable:
            # varBindTableRow:
            #     [(ObjectName(1.3.6.1.2.1.17.7.1.2.2.1.2.5.0.27.144.212.92.45),
            #     Integer(27))]

            data = varBindTableRow[0][0]._value[len(value):]

            vlan = data[0]
            #mac = '%s' % ':'.join([hex(int(i))[2:] for i in data[-6:]])
            mac = '%02x:%02x:%02x:%02x:%02x:%02x' % tuple(map(int, data[-6:]))
            port = varBindTableRow[0][1]
            yield {'vlan': vlan, 'mac': mac, 'port': port}

if __name__ == '__main__':
    try:
        ip = sys.argv[1]
        community = sys.argv[2]
    except IndexError:
        print "PLease run command like:"
        print "python %s <ip> <community>" % __file__
        sys.exit(0)

    for fdb in fetchFdb(ip, community):
        print 'vlan: %(vlan)s mac: %(mac)s port: %(port)s' % (fdb)




import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='snmpdesk',
    version='0.0.9',
    description='Scripts for easy get snmp data',
    author='Svintsov Dmitry',
    author_email='spam@19216801.ru',
    url='http://github.com/uralbash/snmpdesk/',
    keywords = "snmp",
    install_requires=['pysnmp'],
    license='GPL',
    packages=['snmpdesk'],
    long_description=read('README'),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Natural Language :: English',
        'Natural Language :: Russian',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Topic :: System :: Networking :: Monitoring',
        ],
)



import sys
import collections
from pysnmp.entity.rfc3413.oneliner import cmdgen

'''def datafrommib(mib, community, ip):
    value = tuple([int(i) for i in mib.split('.')])
    generator = cmdgen.CommandGenerator()
    comm_data = cmdgen.CommunityData('server', community, 1) # 1 means version SNMP v2c
    transport = cmdgen.UdpTransportTarget((ip, 161))

    real_fun = getattr(generator, 'nextCmd')
    res = (errorIndication, errorStatus, errorIndex, varBindTable)\
            = real_fun(comm_data, transport, value)

    if not errorIndication is None  or errorStatus is True:
           print "Error: %s %s %s %s" % res
           yield None
    else:
        for varBindTableRow in varBindTable:
            # varBindTableRow:
            #   in: [(ObjectName(1.3.6.1.2.1.2.2.1.10.8), Counter32(180283794))]
            data = varBindTableRow[0]
            port = data[0]._value[len(value):]
            octets = data[1]

            yield {'port': port[0], 'octets': octets}
'''

def datafrommib(mib, community, conn):
    value = tuple([int(i) for i in mib.split('.')])
    #res = (errorIndication, errorStatus, errorIndex, varBindTable)\
    #        = real_fun(comm_data, transport, value)
    res = (errorIndication, errorStatus, errorIndex, varBindTable)\
            = conn[3](conn[1], conn[2])

    if not errorIndication is None  or errorStatus is True:
           print "Error: %s %s %s %s" % res
           yield None
    else:
        for varBindTableRow in varBindTable:
            # varBindTableRow:
            #   in: [(ObjectName(1.3.6.1.2.1.2.2.1.10.8), Counter32(180283794))]
            data = varBindTableRow[0]
            port = data[0]._value[len(value):]
            octets = data[1]

            yield {'port': port[0], 'octets': octets}

def load(ip, community):
    # for use snmptool try:
    # In: snmpwalk -c mymypub -v2c <ip> 1.3.6.1.2.1.2.2.1.10.2
    # Out: snmpwalk -c mymypub -v2c <ip> 1.3.6.1.2.1.2.2.1.16.2
    # e.t.c...
    generator = cmdgen.CommandGenerator()
    comm_data = cmdgen.CommunityData('server', community, 1) # 1 means version SNMP v2c
    transport = cmdgen.UdpTransportTarget((ip, 161))
    real_fun = getattr(generator, 'nextCmd')
    conn = (generator, comm_data, transport, real_fun)
    mibs = [('1.3.6.1.2.1.2.2.1.16', 'out'),
            ('1.3.6.1.2.1.2.2.1.10', 'in'),
            ('1.3.6.1.2.1.2.2.1.11', 'ucast'),
            ('1.3.6.1.2.1.2.2.1.12', 'nucast'),
            ('1.3.6.1.2.1.2.2.1.13', 'discards'),
            ('1.3.6.1.2.1.2.2.1.14', 'errors')]

    ports = collections.defaultdict(dict)

    for mib in mibs:
        data = datafrommib(mib[0], community, conn)
        for row in data:
            if row:
                ports[row['port']][mib[1]] = int(row['octets'])
            else:
                return None

    return ports

if __name__ == '__main__':
    try:
        ip = sys.argv[1]
        community = sys.argv[2]
    except IndexError:
        print "Please run command like:"
        print "python %s <ip> <community>" % __file__
        sys.exit(0)
    # == debug ==
    #import profile
    #profile.run("load('%s', '%s')" % (ip, community))
    ports = load(ip, community)
    if ports:
        for key, value in ports.items():
            print key, ('in: %(in)s out: %(out)s ucast: %(ucast)s' +\
                       ' nucast: %(nucast)s discards: %(discards)s' +\
                       ' errors: %(errors)s') % value

